%Tutorial 2: Space-Time SVD, spatial basis, projections and filtering

%This data explores the role of the SVD algorithm in in space-time
%processing. This is a particular factorization of a matrix that arises from
%A result called the spectral theorem. This can be done through interpreting 
%the basis vectors as spatial modes (or shapes), interpreting the principle 
%components in terms of temporal oscillations of those modes. Data can then
%be reduced by filtering using the SVD basis.

    %% BEGIN SECTION 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % 
    %   SECTION 1: Decomposing waves using SVD
    %
    %   Previously, we defined data D = [observations time], where observations
    %   where simultanoeusly sampled data. In space-time analysis, the data 
    %   matrix has the follow dimensions:
    %
    %       D = [space time]
    %
    %   Where the observations are taken over some regular partition of space.
    %   This can be done in any number of dimensions, but due to discrete 
    %   sampling, can still be thought of in one dimension.
    %
    %   The SVD decomposition is a factorization of an arbitrary data D in the
    %   following way:
    %
    %       D = S*T'*E';
    %
    %   Where S(:,k) is the kth spatial eigenvectors, T(k,:) is the kth 
    %   temporal component and E is an energy matrix.
    %
    %   T has the property that T*T' = E, which is a diagonal matrix. This 
    %   property is equivalent to the fact that the temporal components of the 
    %   decomposition are uncorrelated.

    N = 15;
    t = 1:100; 
    [X Y Z] = meshgrid(linspace(0,1.5*pi/2,N),linspace(0,1.5*pi/2,N),linspace(0,6*pi,length(t)));
    data1 = cos(X + Z)+cos(Y + Z);
    
    [X Y Z] = meshgrid(linspace(0,6*pi,N),linspace(0,6*pi/2,N),linspace(0,12*pi,length(t)));
    data2 = cos(X + 3*Z)+cos(Y - 3*Z);
    data = data1 + .25*data2;
    
    s = size(data);
    datatemp = zeros(s(1)*s(2),s(3));
    for k = 1:s(1)
        datatemp(((k-1)*s(1) + 1):k*(s(1)),:) = data(k,:,:);
    end
    [dataSVD11 B2 E] = SVDfilter(datatemp,2);
    [dataSVD22 B2 E] = SVDfilter(datatemp,10);
    dataSVD22 = dataSVD22-dataSVD11;
    
    dataSVD1 = repmat(nan,size(data));
    dataSVD2 = repmat(nan,size(data));
    B = zeros(N,N,N*N);
    for k = 1:s(1)
        dataSVD1(k,:,:) = dataSVD11(((k-1)*s(1) + 1):k*(s(1)),:);
        dataSVD2(k,:,:) = dataSVD22(((k-1)*s(1) + 1):k*(s(1)),:);
    end
    for n = 1:N*N
        temp = zeros(N,N);
        for k = 1:N;
            temp(:,k) = B2(((k-1)*s(1) + 1):k*(s(1)),n);
        end
        B(:,:,n) = temp;
    end
    
    figure;
    setP([5 384 1654 697]);
    done = 0;
    k = 1;
    while(~done)
        subplot(2,4,1);
            imagesc(data1(:,:,k));
            axis square; setXTL; setYTL;
            title('Wave1');
        subplot(2,4,2);
            imagesc(data2(:,:,k));
            axis square; setXTL; setYTL;
            title('Wave2');
        subplot(1,2,2);
            imagesc(data(:,:,k));
            axis square; setXTL; setYTL;
            title('Wave1 + Wave2');
        subplot(2,4,5);
            imagesc(dataSVD1(:,:,k));
            axis square; setXTL; setYTL;
            title('SVDFilter1 : 1-2');
        subplot(2,4,6);
            imagesc(dataSVD2(:,:,k));
            axis square; setXTL; setYTL;
            title('SVDFilter1 : 3-4');
        pause(.025);
        k = mod(k,length(t)) + 1;
    end
    
    subplot(2,2,1); 
        imagesc(B(:,:,1)); 
    subplot(2,2,2); 
        imagesc(B(:,:,2)); 
    subplot(2,2,3); 
        imagesc(B(:,:,3)); 
    subplot(2,2,4); 
        imagesc(B(:,:,4));
    
    
    
    

    %%  QUESTIONS:

    
%% END SECTION 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%